<?php

//output(): Outputs a message to both stdio and to the log file, if logging is on.
function output($msg){
	$Bot=null;
	$echoit=true;
	if (isset($GLOBALS['Bot']) && is_object($GLOBALS['Bot'])){
		$Bot=&$GLOBALS['Bot'];
	}
	elseif (isset($GLOBALS['_Bot']) && is_object($GLOBALS['_Bot'])){
		$Bot=&$GLOBALS['_Bot'];
		$echoit=false;
	}

	if (is_null($Bot)){
		return false;
	}
	$msg=trim($msg);
	if ($Bot->getCfg('output_logging')=='true'){
		$logFile=trim($Bot->getCfg('output_log_prefix'));
		$logFile.=date("m-d-y-H-i-s",$Bot->getInfo('log_date'));
		$logFile.='.log';
		if (!file_exists($logFile)){
			touch($logFile);
			chmod($logFile, 0777);
		}
		$fp=fopen($logFile, 'a');
		if ($fp){
			fputs($fp, $msg."\n");
		}
		fclose($fp);
	}
	if ($echoit){
		echo $msg."\n";
	}
}


//getData(): Gets the data from a stream.  I may add more to this, so I decided to make it a function.
function getData($con, $amount){
	$line=fgets($con, $amount);
	$line=trim($line);
	return $line;
}


//toArray(): Creates an array out of a string such as ['Wiser','phpB','Wiser-Than-You']
function toArray($string){
	$string=trim($string);
	$parts=array();
	if (empty($string)){
		return array();
	}
	if (substr($string, 0, 2) != '["' && substr($string, 0, 2) != "['"){
		return array($string);
	}
	if ($string=='[""]' || $string=="['']"){
		return array();
	}

	$style=substr($string, 1,1);

	$string=str_replace("[$style",chr(1).chr(2),$string);
	$string=str_replace("$style,$style",chr(1).chr(2),$string);
	$string=str_replace("$style, $style",chr(1).chr(2),$string);
	$string=str_replace("$style , $style",chr(1).chr(2),$string);
	$string=str_replace("$style ,$style",chr(1).chr(2),$string);
	$string=str_replace("$style]",chr(1).chr(2),$string);

	$string=substr($string, 2, -2);

	$string=str_replace("\\$style",$style,$string);

	return explode(chr(1).chr(2), $string);
}

function toArrayString($array){
	if (count($array) == 0){
		return '';
	}
	$return='[';
	foreach ($array as $value){
		$value=str_replace("'","\\'",$value);
		$return .= "'{$value}',";
	}
	$return=substr($return, 0, -1).']';
	return $return;
}
//msgServer(): Used to send commands to the IRC server
function msgServer($msg){
	$msg=trim($msg);
	output($msg);
	fputs($GLOBALS['__IRC_CONNECTION'], $msg."\r\n");
}

//getBreakPattern(): This takes the message recieved from the server and figures out which type of message it is.
//After it figures it out, it returns an array with key 0 being the type, key 1 the break pattern, and key 2 the matches
function getBreakPattern($rawLine){
	$patterns=array(
			'CTCPMSG'=>'^:(.[^!]*)!(.[^@]*)@([\w.-]+) ([A-Z*]+) (.[^ ]+) ?:'.chr(1).'(.*)'.chr(1).'$',
			'NORMALMSG'=>'^:(.[^!]*)!(.[^@]*)@([\w.-]+) ([A-Z*]+) (:?.[^ ]+) ?(:?.*)$',
			'SERVERCODE'=>'^:(.[^ ]*) (\d{3}) (.[^ ]*) (.*)$',
			'SERVERNOTICE'=>'^:(.[^ ]*) ([A-Z*]+) (.[^ ]+) (:?.*)$',
			'SELFMODE'=>'^:(.*) MODE (.*) :(.*)$'
		);

	foreach ($patterns as $key=>$val){
		$matches=null;
		if (preg_match("/$val/",$rawLine, $matches)){
			return array($key, $val, $matches);
		}
	}
	return null;
}

//FormatSize(): This can be used to format the file size of something before sending it to IRC, if need be.
function FormatSize($size){
	$tmp2=$size/1024;
	if ($tmp2<1){
		$size="{$size} bytes";
	}
	else {
		$tmp3=$tmp2/1024;
		if ($tmp3<1){
			$size=round($tmp2, 2)." Kb";
		}
		else {
			$tmp4=$tmp3/1024;
			if ($tmp4<1){
				$size=round($tmp3, 2)." Mb";
			}
			else {
				$size=round($tmp4, 2)." Gb";
			}
		}
	}
	return $size;
}

//resetDb(): Resets the database so that everything is as it should be for a startup.
function resetDb(){
	//Make sure everyone is markes as offline and logged out.
	db_query("UPDATE users SET loggedin='N', online='N', InheritedModes=''");
	
	//Make sure the channels table is empty
	db_query("DELETE FROM channels");

	//Make sure the chanusers table is empty
	db_query("DELETE FROM chanusers");

	//Make sure the info table is empty
	db_query("DELETE FROM info");
	return 1;
}

//Makes a string regex safe so that it may be used in a ereg or preg
function regexSafe($txt){
	$txt=addslashes($txt);
	$txt=str_replace("]","\\]",$txt);
	$txt=str_replace("[","\\[",$txt);
	$txt=str_replace(".","[.]",$txt);
	$txt=str_replace("?","[?]",$txt);
	$txt=str_replace("^","\\^",$txt);
	$txt=str_replace("$","[$]",$txt);
	$txt=str_replace("*","[*]",$txt);
	$txt=str_replace("(","[(]",$txt);
	$txt=str_replace(")","[)]",$txt);
	$txt=str_replace("+","[+]",$txt);
	$txt=str_replace("|","[|]",$txt);
	return $txt;
}

function makeDataArray($matches){
	list($Type, $Pattern, $Matches)=$matches;
	//All the data will be stored into the associative array $Info
	$Info=array(
			'NICK'=>null,
			'IDENT'=>null,
			'HOST'=>null,
			'TYPE'=>null,
			'CODE'=>null,
			'DEST'=>null,
			'MSG'=>null,
			'RAW'=>$Matches[0],
			'PATTERN'=>$Pattern,
			'BREAK_TYPE'=>$Type,
			'RUNABLE'=>true,
			'EMBEDDED'=>false
		);

	//Decide how to handle the message data and how to re-arrange it (to make it eaiser to access)
	switch ($Type){
		case 'SERVERCODE':{
			$Info['NICK']='';
			$Info['IDENT']='';
			$Info['HOST']=$Matches[1];
			$Info['TYPE']='';
			$Info['CODE']=$Matches[2];
			$Info['DEST']=$Matches[3];
			$Info['MSG']=$Matches[4];

			if (strlen($Info['MSG']) > 0 && $Info['MSG']{0}==':'){
				$Info['MSG']=substr($Info['MSG'], 1);
			}

			break;
		}
		case 'SERVERNOTICE':{
			$Info['NICK']='';
			$Info['IDENT']='';
			$Info['HOST']=$Matches[1];
			$Info['TYPE']=$Matches[2];
			$Info['CODE']='000';
			$Info['DEST']=$Matches[3];
			$Info['MSG']=$Matches[4];

			if (strlen($Info['MSG']) > 0 && $Info['MSG']{0}==':'){
				$Info['MSG']=substr($Info['MSG'], 1);
			}

			break;
		}
		case 'CTCPMSG':
			$Info['MSG']='~';
		case 'NORMALMSG':{
			$Info['NICK']=$Matches[1];
			$Info['IDENT']=$Matches[2];
			$Info['HOST']=$Matches[3];
			$Info['TYPE']=$Matches[4];
			$Info['CODE']='000';
			$Info['DEST']=$Matches[5];
			$Info['MSG'].=$Matches[6];

			if (strlen($Info['MSG']) > 0 && $Info['MSG']{0}==':'){
				$Info['MSG']=substr($Info['MSG'], 1);
			}
			break;
		}
		case 'SELFMODE':{
			$Info['NICK']=$Matches[1];
			$Info['IDENT']='';
			$Info['HOST']='';
			$Info['TYPE']='MODE';
			$Info['CODE']='000';
			$Info['DEST']=$Matches[2];
			$Info['MSG']=$Matches[3];

			if (strlen($Info['MSG']) > 0 && $Info['MSG']{0}==':'){
				$Info['MSG']=substr($Info['MSG'], 1);
			}
			break;
		}
	}
	return $Info;
}

function stripCodes($line){
	$line=str_replace(chr(1),'',$line);
	$line=str_replace(chr(2),'',$line);
	$line=str_replace(chr(15),'',$line);
	$line=str_replace(chr(31),'',$line);
	$line=str_replace(chr(22),'',$line);
	$line=preg_replace('/'.chr(3).'\d(,\d)?/','',$line);
	return $line;
}

function timeFormat($dif){
	$Aday=(60*60*24);
	$Ahour=(60*60);
	$Amin=60;
	$Ayear=$Aday*365;

	$years=(int)($dif/$Ayear);
	if ($years >= 1){
		$dif-=$Ayear*$years;
		$years.=($years==1)?" year":" years";
		$years.=" ";
	}
	else {
		$years='';
	}
	$days=(int)($dif/$Aday);
	if ($days >= 1){
		$dif-=$Aday*$days;
		$days.=($days==1)?" day":" days";
		$days.=" ";
	}
	else {
		$days='';
	}
	$hours=(int)($dif/$Ahour);
	if ($hours >= 1){
		$dif-=$Ahour*$hours;
		$hours.=($hours==1)?" hour":" hours";
		$hours.=" ";
	}
	else {
		$hours='';
	}
	$mins=(int)($dif/$Amin);
	if ($mins>=1){
		$dif-=$Amin*$mins;
		$mins.=($mins==1)?" minute":" minutes";
		$mins.=" ";
	}
	else {
		$mins='';
	}
	$seconds=($dif==1)?"$dif second":"$dif seconds";
	$uptime="$years$days$hours$mins$seconds";
	return $uptime;
}

function stri_replace($key, $replacement, $string){
	$pos=0;
	$keyLower=strtolower($key);
	$stringLower=strtolower($string);

	while (($pos=strpos($stringLower, $keyLower, $pos))!==false){
		$posEnd=$pos+strlen($keyLower);
		if ($posEnd > strlen($stringLower)){
			$posEnd=strlen($stringLower);
		}

		$tmp=substr($string, 0, $pos);
		$tmp.=$replacement;
		$tmp.=substr($string, $posEnd);

		$string=$tmp;
		$stringLower=strtolower($string);
		$pos+=strlen($replacement);
	}
	return $string;
}

function DebugPrint($msg){
	if (isset($_POST['debug'])){
		output($msg."\n\tFile:".$_SERVER['PHP_SELF']);
	}
}
?>
